package br.edu.fei.dcc.raciocinioespacial.editor.shapes;

import java.util.EventObject;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.draw2d.ShortestPathConnectionRouter;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.dnd.TemplateTransferDragSourceListener;
import org.eclipse.gef.dnd.TemplateTransferDropTargetListener;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.requests.SimpleFactory;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.palette.PaletteViewer;
import org.eclipse.gef.ui.palette.PaletteViewerProvider;
import org.eclipse.gef.ui.palette.FlyoutPaletteComposite.FlyoutPreferences;
import org.eclipse.gef.ui.parts.ContentOutlinePage;
import org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette;
import org.eclipse.gef.ui.parts.GraphicalViewerKeyHandler;
import org.eclipse.gef.ui.parts.TreeViewer;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.util.TransferDropTargetListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.IPageSite;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

import br.edu.fei.dcc.raciocinioespacial.editor.commands.Transformacao;
import br.edu.fei.dcc.raciocinioespacial.editor.commands.Utils;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.ShapesDiagram;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.palette.ShapesEditorPaletteFactory;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.parts.ShapesEditPartFactory;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.parts.ShapesTreeEditPartFactory;
import br.edu.fei.dcc.raciocinioespacial.simulador.persistencia.PersistenciaSimulacao;

/**
 * Graphic Editor to edit '.stv' files.
 */
public class ShapesEditor extends GraphicalEditorWithFlyoutPalette 
{

	/** Identificador do editor */
	public static final String ID = "br.edu.fei.dcc.raciocinioespacial.shapes.editor.gef";

	/** Model root for editing. */
	private ShapesDiagram diagram;
	/** Palette containing the tools and shapes. */
	private static PaletteRoot paletteModel;

	/** Create an instance of ShapesEditor. Method called by Workspace. */
	public ShapesEditor() {
		super.setEditDomain(new DefaultEditDomain(this));
	}

	/**
	 * Configura a visualizao do editor antes de receber o contedo.
	 * <p>Este  o lugar para escolher o RootEditPart e o EditPartFactory apropriado.
	 * para o editor. O RootEditPart determina o comportamento da "work-area" do editor.
	 * Por exemplo, GEF inclui as partes de edio "Zoomablle" e "Scrollable". 
	 * O EditPartFactory mapeia os elementos do modelo para as partes de edio (controllers).</p>
	 * @see org.eclipse.gef.ui.parts.GraphicalEditor#configureGraphicalViewer()
	 */
	@Override
	protected void configureGraphicalViewer() {
		super.configureGraphicalViewer();

		GraphicalViewer viewer = getGraphicalViewer();
		viewer.setEditPartFactory(new ShapesEditPartFactory());
		viewer.setRootEditPart(new ScalableFreeformRootEditPart());
		viewer.setKeyHandler(new GraphicalViewerKeyHandler(viewer));

		// Configura o contexto de menu.
		ContextMenuProvider cmProvider =
			new ShapesEditorContextMenuProvider(viewer, getActionRegistry(), this.diagram);
		viewer.setContextMenu(cmProvider);
		getSite().registerContextMenu(cmProvider, viewer);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.ui.parts.GraphicalEditor#commandStackChanged(java.util.EventObject)
	 */
	@Override
	public void commandStackChanged(EventObject event) {
		firePropertyChange(IEditorPart.PROP_DIRTY);
		super.commandStackChanged(event);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette#createPaletteViewerProvider()
	 */
	@Override
	protected PaletteViewerProvider createPaletteViewerProvider() {
		return new PaletteViewerProvider(getEditDomain()) {
			@Override
			protected void configurePaletteViewer(PaletteViewer viewer) {
				super.configurePaletteViewer(viewer);
				/* Cria o listener para o visualizador da paleta.
				 * Mapeia aes como o drag and drop.
				 * @see ShapesEditor#createTransferDropTargetListener()
				 */
				viewer.addDragSourceListener(new TemplateTransferDragSourceListener(viewer));
			}
		};
	}

	/**
	 * Cria o listener para rea de transferncia do drop. 
	 * Quando usado com  a ferramenta CombinedTemplateCreationEntry na paleta
	 * isto habilitar o ElementModelo de criao a partir do Drag da paleta.
	 * @see #createPaletteViewerProvider()
	 */
	private TransferDropTargetListener createTransferDropTargetListener() {
		return new TemplateTransferDropTargetListener(getGraphicalViewer()) {
			@Override
			protected CreationFactory getFactory(Object template) {
				return new SimpleFactory((Class<?>) template);
			}
		};
	}

	/**
	 * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void doSave(IProgressMonitor monitor) {
		this.doSaveAs();
	}

	/**
	 * @see org.eclipse.ui.ISaveablePart#doSaveAs()
	 */
	@Override
	public void doSaveAs() {
		String path = Utils.openFileDialog(this.getEditorSite().getShell(), "Save simulation", "*.stv", SWT.SAVE);
		
		if (path != null)
		{
			PersistenciaSimulacao simulacaoSalva = new PersistenciaSimulacao(path, new Transformacao(this.diagram).getSimulacao());
			try{
				simulacaoSalva.salvar();
				getCommandStack().markSaveLocation();
				MessageDialog.openInformation(this.getEditorSite().getShell(), "Success", "Simulation saved sucessfully.");
			}catch(Exception e)
			{
				e.printStackTrace();
				MessageDialog.openError(this.getEditorSite().getShell(), "Error", "Error while saving the simulation.\n\n" + e.getMessage());
			}
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public Object getAdapter(Class type) {
		if (type == IContentOutlinePage.class)
			return new ShapesOutlinePage(new TreeViewer());
		return super.getAdapter(type);
	}

	public ShapesDiagram getModel() {
		return diagram;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette#getPalettePreferences()
	 */
	@Override
	protected FlyoutPreferences getPalettePreferences() {
		return ShapesEditorPaletteFactory.createPalettePreferences();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette#getPaletteRoot()
	 */
	@Override
	protected PaletteRoot getPaletteRoot() {
		if (paletteModel == null)
			paletteModel = ShapesEditorPaletteFactory.createPalette();
		return paletteModel;
	}

	@SuppressWarnings("unused")
	private void handleLoadException(Exception e) {
		System.err.println("Loading Error. Use default model"); //$NON-NLS-1$
		e.printStackTrace();
		diagram = new ShapesDiagram();
	}

	/**
	 * Configura o contedo inicial da aplicao (depois da criao).
	 * @see org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette#initializeGraphicalViewer()
	 */
	@Override
	protected void initializeGraphicalViewer() {
		super.initializeGraphicalViewer();
		GraphicalViewer viewer = getGraphicalViewer();
		viewer.setContents(getModel()); // define o contedo do editor.

		// Adiciona o ShortestPathConnectionRouter
		ScalableFreeformRootEditPart root = 
			(ScalableFreeformRootEditPart)viewer.getRootEditPart();
		ConnectionLayer connLayer =
			(ConnectionLayer)root.getLayer(LayerConstants.CONNECTION_LAYER);
		GraphicalEditPart contentEditPart = (GraphicalEditPart)root.getContents();
		ShortestPathConnectionRouter router = 
			new ShortestPathConnectionRouter(contentEditPart.getFigure());
		connLayer.setConnectionRouter(router);
		//contentEditPart.getContentPane().addLayoutListener(router.getLayoutListener());

		// Mapeia por reas excludas.
		viewer.addDropTargetListener(createTransferDropTargetListener());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISaveablePart#isSaveAsAllowed()
	 */
	@Override
	public boolean isSaveAsAllowed() {
		return true;
	}


	/**
	 * Utiliza o ShapesEditorInput para servir com um editor vazio.
	 * Isto serve para definir o cenrio inicial do editor de formas.
	 * @see org.eclipse.ui.part.EditorPart#setInput(org.eclipse.ui.IEditorInput)
	 */
	@Override
	protected void setInput(IEditorInput input) {
		super.setInput(input);
		ShapesEditorInput shapesInput = ((ShapesEditorInput) input);
		diagram = shapesInput.getShapesDiagram();
		setPartName(shapesInput.getName());
	}

	/**
	 * Cria a parte "Outline" para visualizao dos objetos inseridos no editor.
	 */
	public class ShapesOutlinePage extends ContentOutlinePage {	
		/**
		 * Cria uma  nova pgina "Outline" para o editor de formas.
		 * @param viewer uma viso (instncia de TreeViewer) usado para a pgina "Outline"
		 * @throws IllegalArgumentException se o editor for nulo.
		 */

		public ShapesOutlinePage(EditPartViewer viewer) {
			super(viewer);
		}

		/* (non-Javadoc)
		 * @see org.eclipse.ui.part.IPage#createControl(org.eclipse.swt.widgets.Composite)
		 */
		@Override
		public void createControl(Composite parent) {
			// Cria a viso Outline para o editor.
			getViewer().createControl(parent);
			// Configura a viso Outline.
			getViewer().setEditDomain(getEditDomain());
			getViewer().setEditPartFactory(new ShapesTreeEditPartFactory());
			// Configura e adiciona um menu para a viso.
			ContextMenuProvider cmProvider = new ShapesEditorContextMenuProvider(
					getViewer(), getActionRegistry(), diagram); 
			getViewer().setContextMenu(cmProvider);
			getSite().registerContextMenu(
					"br.edu.fei.dcc.raciocinioespacial.editor.shapes.ShapesEditorContextMenuProvider",
					cmProvider, getSite().getSelectionProvider());		
			// Insere Outline na viso.
			getSelectionSynchronizer().addViewer(getViewer());
			// Inicializa a viso Outline com o modelo.
			getViewer().setContents(getModel());
			// Exibe a viso "Outline"
		}

		/* (non-Javadoc)
		 * @see org.eclipse.ui.part.IPage#dispose()
		 */
		@Override
		public void dispose() {
			// Remove Outline da viso.
			getSelectionSynchronizer().removeViewer(getViewer());
			// Dispose
			super.dispose();
		}

		/* (non-Javadoc)
		 * @see org.eclipse.ui.part.IPage#getControl()
		 */
		@Override
		public Control getControl() {
			return getViewer().getControl();
		}

		/**
		 * @see org.eclipse.ui.part.IPageBookViewPage#init(org.eclipse.ui.part.IPageSite)
		 */
		@Override
		public void init(IPageSite pageSite) {
			super.init(pageSite);
			ActionRegistry registry = getActionRegistry();
			IActionBars bars = pageSite.getActionBars();
			String id = ActionFactory.UNDO.getId();
			bars.setGlobalActionHandler(id, registry.getAction(id));
			id = ActionFactory.REDO.getId();
			bars.setGlobalActionHandler(id, registry.getAction(id));
			id = ActionFactory.DELETE.getId();
			bars.setGlobalActionHandler(id, registry.getAction(id));
		}
	}
}