package br.edu.fei.dcc.raciocinioespacial.editor.shapes.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.editpolicies.RootComponentEditPolicy;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateRequest;

import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Agent2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Pedestrian2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Shape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Street2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.elements.Vehicle2DShape;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.ModelElement;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.ShapesDiagram;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.command.ShapeCreateCommand;
import br.edu.fei.dcc.raciocinioespacial.editor.shapes.model.command.ShapeSetConstraintCommand;

/**
 * EditPart for an instance of ShapesDiagram.
 * <p>It's like a main diagram container, the white area where the
 * objects are stored.</p>
 */
class DiagramEditPart extends AbstractGraphicalEditPart implements PropertyChangeListener  {


	public void activate() {
		if (!isActive()) {
			super.activate();
			((ModelElement) getModel()).addPropertyChangeListener(this);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// desabilita a remoo desta edit part pelo seu ancestral.
		this.installEditPolicy(EditPolicy.COMPONENT_ROLE, new RootComponentEditPolicy());
		// handles mudanas (como movimentao e alterao de tamanho) dos elementos do modelo
		// e cria novos elementos do modelo.
		this.installEditPolicy(EditPolicy.LAYOUT_ROLE,  new ShapesXYLayoutEditPolicy());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		Figure f = new FreeformLayer();
		f.setBorder(new MarginBorder(3));
		f.setLayoutManager(new FreeformLayout());
		return f;
	}

	/**
	 * Aps a desativao,  desanexado do model element como um listener de modificaes das propriedades.
	 */
	public void deactivate() {
		if (isActive()) {
			super.deactivate();
			((ModelElement) getModel()).removePropertyChangeListener(this);
		}
	}

	private ShapesDiagram getCastedModel() {
		return (ShapesDiagram) getModel();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	protected List getModelChildren() {
		return getCastedModel().getChildren(); // return a list of shapes
	}

	/* (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		String prop = evt.getPropertyName();
		// Os eventos dessas propriedades so disparadas quando as formas/Shapes so adicionados ou removidos 
		// da instncia de ShapeDiagram e precisam causar uma chamada refreshChildren()
		// para atualizar o contedo do diagrama.
		if (ShapesDiagram.CHILD_ADDED_PROP.equals(prop)
				|| ShapesDiagram.CHILD_REMOVED_PROP.equals(prop)) {
			refreshChildren();
		}
	}
	
	public void renameChild(Shape x)
	{
		this.registerVisuals();
		
	}

	/**
	 * Poltica de edio para a figura usada nesse edit part.
	 * Classes que herdam XYLayoutEditPolicy podem ser usadas em XYLayout.
	 */
	private class ShapesXYLayoutEditPolicy extends XYLayoutEditPolicy {

		/* (non-Javadoc)
		 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createAddCommand(org.eclipse.gef.EditPart, java.lang.Object)
		 */
		protected Command createAddCommand(EditPart child, Object constraint) {
			return null;
		}


		/* (non-Javadoc)
		 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.requests.ChangeBoundsRequest, org.eclipse.gef.EditPart, java.lang.Object)
		 */
		protected Command createChangeConstraintCommand(ChangeBoundsRequest request,
				EditPart child, Object constraint) {
			if ((child instanceof ShapeEditPart) && 
					(constraint instanceof Rectangle) &&
					(! (child.getModel() instanceof Street2DShape))) {
				// Retorna o comando que pode mover ou redimensionar o objeto.
				return new ShapeSetConstraintCommand(
						(Shape) child.getModel(), request, (Rectangle) constraint);
			}
			return super.createChangeConstraintCommand(request, child, constraint);
		}

		/* (non-Javadoc)
		 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart, java.lang.Object)
		 */
		protected Command createChangeConstraintCommand(EditPart child,
				Object constraint) {
			// not used in this example
			return null;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
		 */
		protected Command getCreateCommand(CreateRequest request) {
			Object childClass = request.getNewObjectType();
			if (childClass == Pedestrian2DShape.class || childClass == Agent2DShape.class || childClass == Vehicle2DShape.class) {
				// return a command that can add a Shape to a ShapesDiagram 
				return new ShapeCreateCommand(DiagramEditPart.this.getCastedModel(), request);
			}
			return null;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getDeleteDependantCommand(org.eclipse.gef.Request)
		 */
		protected Command getDeleteDependantCommand(Request request) {
			// not used in this example
			return null;
		}
	}
}